package com.anpgames.beans.application;

import java.util.HashMap;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.annotation.PostConstruct;
import javax.faces.bean.ApplicationScoped;
import javax.faces.bean.ManagedBean;

import com.anpgames.beans.session.LocaleMB.LocaleType;



@ManagedBean
@ApplicationScoped
public class MessagesSupportMB {
	private HashMap<Bundle, ResourceBundle>[] resourceBundles;
	
	/**
	 * Typ wyliczeniowy określający paczkę komunikatów.
	 */
	public static enum Bundle {
		/** Komunikaty dotyczące całej aplikacji. */
		MSGS_APP,
		/** Komunikaty dotyczące modułu administracyjnego. */
		MSGS_ADMIN
	}
	
	/**
	 * Typ wyliczeniowy określający rodzaj komunikatu.
	 */
	public static enum MsgType {
		DEFAULT,
		WARNING,
		ERROR,
		SUCCESS
	}
	
	
	/**
	 * Inicjalizacja tablicy, której elementami są mapy paczek komunikatów.
	 * Odwołania do tablicy następują za pomocą typu wyliczeniowego <code>LocaleType</code> określającego język.
	 * Kluczem mapy jest typ paczki komunikatów <code>Bundle</code>, wartością natomiast jest już sama paczka <code>ResourceBundle</code>.
	 * Dzięki takiemu rozwiązaniu paczki cały czas przechowywane są w pamięci, a każde odwołanie do konkretnego komunikatu nie powoduje sięgania do plików.
	 */
	@SuppressWarnings("unchecked")
	@PostConstruct
	public void init() {
		resourceBundles = (HashMap<Bundle, ResourceBundle>[]) new HashMap[LocaleType.values().length];
		initResourceBundleMap(LocaleType.PL);
		initResourceBundleMap(LocaleType.EN);
	}
	
	/**
	 * Metoda inicjalizująca mapę z paczkami komunikatów.
	 * Pod konkretnym indeksem tablicy inicjalizowana jest mapa paczek komunikatów.
	 * Następnie w mapie umieszczanę są elementy klasy <code>ResourceBundle</code> dla każdego rodzaju paczki.
	 * @param localeType typ określający dla jakiego języka należy zainicjalizować mapę z paczkami komunikatów
	 */
	public void initResourceBundleMap(LocaleType localeType) {
		int index = localeType.ordinal();
		ResourceBundle rb = null;
		resourceBundles[index] = new HashMap<MessagesSupportMB.Bundle, ResourceBundle>();
		
		rb = ResourceBundle.getBundle("com.anpgames.bundles.messages", new Locale(localeType.toString().toLowerCase()));
		resourceBundles[index].put(Bundle.MSGS_APP, rb);
		
		rb = ResourceBundle.getBundle("com.anpgames.modules.admin.bundles.messages", new Locale(localeType.toString().toLowerCase()));
		resourceBundles[index].put(Bundle.MSGS_ADMIN, rb);
	}
	
	/**
	 * Metoda zwracająca komunikat w odpowiednim języku na modstawie klucza.
	 * @param localeType typ określający język
	 * @param bundle typ określający paczkę komunikatów
	 * @param key klucz określający, który komunikat należy pobrać z paczki
	 * @return komunikat
	 */
	public String getMessage(LocaleType localeType, Bundle bundle, String key){
		return resourceBundles[localeType.ordinal()].get(bundle).getString(key);
	}
	
	/**
	 * Metoda zwracająca nazwę klasy CSS dla odpowiedniego typu komunikatu.
	 * @param msgType typ komunikatu
	 * @return nazwa klasy CSS
	 */
	public static String getMessageCSSClass(MsgType msgType) {
		switch (msgType) {
		case DEFAULT:
			return "respMsgDefault";
		case WARNING:
			return "respMsgWarning";
		case ERROR:
			return "respMsgError";
		case SUCCESS:
			return "respMsgSuccess";
		}
		return null;
	}

}
